@@ -760,16 +760,10 @@ get_alias_quirk(struct usb_device *dev, unsigned int id)
  */
 static int try_to_register_card(struct snd_usb_audio *chip, int ifnum)
 {
-	struct usb_interface *iface;
-
 	if (check_delayed_register_option(chip) == ifnum ||
-	    chip->last_iface == ifnum)
+	    chip->last_iface == ifnum ||
+	    usb_interface_claimed(usb_ifnum_to_if(chip->dev, chip->last_iface)))
 		return snd_card_register(chip->card);
-
-	iface = usb_ifnum_to_if(chip->dev, chip->last_iface);
-	if (iface && usb_interface_claimed(iface))
-		return snd_card_register(chip->card);
-
 	return 0;
 }
 
